==> org-lsst-ccs-subsystem-refrig-main-4.3.3-sources.jar: org/lsst/ccs/subsystem/refrig/RefrigActionManager.java <==
package org.lsst.ccs.subsystem.refrig;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.Agent;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.bus.data.Alert;
import org.lsst.ccs.bus.data.RaisedAlertHistory;
import org.lsst.ccs.bus.states.AlertState;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.messaging.AgentPresenceListener;
import org.lsst.ccs.services.AgentPropertiesService;
import org.lsst.ccs.services.alert.AlertEvent;
import org.lsst.ccs.services.alert.AlertListener;
import org.lsst.ccs.services.alert.AlertService;
import org.lsst.ccs.subsystem.refrig.data.RefrigAction;

/**
 * This class manages the refrig actions in response to Alert payloads.
 *
 * It uses the AgentPresenceManager to detect which Agents on the buses might be
 * sending Alert payloads to trigger actions.
 *
 * @author The LSST CCS Team
 */
public class RefrigActionManager implements HasLifecycle, AlertListener, AgentPresenceListener {

    private static final Logger LOG = Logger.getLogger(RefrigActionManager.class.getName());

    @LookupField(strategy = LookupField.Strategy.TOP)
    private Agent agent;
    @LookupField(strategy = LookupField.Strategy.TREE)
    private AgentPropertiesService propertiesService;
    @LookupField(strategy = LookupField.Strategy.TREE)
    private AlertService alertService;
    @LookupField(strategy = LookupField.Strategy.TREE)
    private final Map<String, ColdCompressor> coldComps = new HashMap<>();
    @LookupField(strategy = LookupField.Strategy.TREE)
    private final Map<String, CryoCompressor> cryoComps = new HashMap<>();

    private final Set<String> origins = new ConcurrentSkipListSet<>();  // Client (originating) agents
    private final Set<String> clearOrigins = new ConcurrentSkipListSet<>();  // Agents to be cleared when disconnect processed
    private final Map<String, Set<String>>[] active = new Map[RefrigAction.Action.values().length];  // Active actions with agents

    /**
     *  Constructor.
     */
    public RefrigActionManager()
    {
        for (int j = 0; j < active.length; j++) {
            active[j] = new HashMap<>();
        }
    }

    /**
     *  Build phase.
     */
    @Override
    public void build()
    {
        active[RefrigAction.Action.STOP_ALL_COLD_COMPS.ordinal()].put("", new ConcurrentSkipListSet<>());
        for (String name : coldComps.keySet()) {
            active[RefrigAction.Action.STOP_COLD_COMP.ordinal()].put(name, new ConcurrentSkipListSet<>());
        }
        active[RefrigAction.Action.STOP_ALL_CRYO_COMPS.ordinal()].put("", new ConcurrentSkipListSet<>());
        for (String name : cryoComps.keySet()) {
            active[RefrigAction.Action.STOP_CRYO_COMP.ordinal()].put(name, new ConcurrentSkipListSet<>());
        }
    }

    /**
     *  Init phase.
     */
    @Override
    public void init()
    {
        // Set a property to define agent as MPM action manager.
        propertiesService.setAgentProperty(RefrigAction.getManagerName(), "");
        // Start alert listening
        alertService.startStatusAlertListening((info) -> {
            return info.isAgentWorkerOrService();
        });
        // Agent presence listener
        agent.getMessagingAccess().getAgentPresenceManager().addAgentPresenceListener(this);
    }

    /**
     *  Start phase.
     */
    @Override
    public void start()
    {
        //Alert listener responsible to listen for Alert emergency payloads
        //published by the origins maintained by the OriginManager
        alertService.addListener(this);
    }

    /**
     *  Shutdown phase.
     */
    @Override
    public void shutdown()
    {
        //Remove agent presence and alert listeners
        agent.getMessagingAccess().getAgentPresenceManager().removeAgentPresenceListener(this);
        alertService.removeListener(this);
    }

    /**
     *  Agent connection handler.
     * 
     *  @param  agents  Connecting agents
     */
    @Override
    public void connecting(AgentInfo... agents)  // Must use this, not "connected"
    {
        for (AgentInfo ai : agents) {
            if (ai.hasAgentProperty(RefrigAction.getClientName())) {
                LOG.log(Level.INFO, "Adding agent {0} to list of origins for payload {1}",
                        new Object[]{ai.getName(), RefrigAction.getName()});
                origins.add(ai.getName());
                clearOrigins.remove(ai.getName());
            }
        }
    }

    /**
     *  Agent disconnection handler.
     * 
     *  @param  agents  Disconnected agents
     */
    @Override
    public void disconnected(AgentInfo... agents)
    {
        for (AgentInfo ai : agents) {
            if (ai.hasAgentProperty(RefrigAction.getClientName())) {
                LOG.log(Level.INFO, "Removing agent {0} from list of origins for payload {1}",
                        new Object[]{ai.getName(), RefrigAction.getName()});
                removeOrigin(ai.getName());
            }
        }
    }

    /**
     *  Raised alert handler.
     * 
     *  @param  event  The alert event
     */
    @Override
    public void onAlert(AlertEvent event)
    {
        String origin = event.getSource();
        //LOG.log(Level.INFO, "onAlert called: origin = {0}, type = {1}", new Object[]{origin, event.getType()});
        if (origins.contains(origin)) {
            switch (event.getType()) {
            case AGENT_CONNECTION:
                for (RaisedAlertHistory alertHistory : event.getSummary().getAllRaisedAlertHistories()) {
                    processAlert(origin, alertHistory.getLatestAlert(), alertHistory.getLatestAlertState());                    
                }
                break;
            case AGENT_DISCONNECTION:
                removeOrigin(origin);
                for (RefrigAction.Action action : RefrigAction.Action.values()) {
                    for (Map.Entry<String, Set<String>> entry : active[action.ordinal()].entrySet()) {
                        if (entry.getValue().remove(origin) && entry.getValue().isEmpty()) {
                            performAction(action, entry.getKey(), false);
                        }
                    }
                }
                break;
            case ALERT_RAISED:
                processAlert(origin, event.getAlert(), event.getLevel());
                break;
            }
        }

    }

    /**
     *  Processes an alert.
     * 
     *  @param  origin  Name of originating agent
     *  @param  alert  The alert
     *  @param  state  The alert state (ALARM, WARNING, NOMINAL)
     */
    private synchronized void processAlert(String origin, Alert alert, AlertState state)
    {
        RefrigAction action = RefrigAction.getData(alert);
        if (action != null) {
            LOG.log(Level.INFO, "Received {0} level alert from {1}: action = ({2}, {3})",
                    new Object[]{state, origin, action.action, action.compName});
            Set orgSet = active[action.action.ordinal()].get(action.compName);
            if (orgSet == null) return;
            boolean on = state == AlertState.ALARM;
            if (on) {
                orgSet.add(origin);
            }
            else {
                orgSet.remove(origin);
                if (!orgSet.isEmpty()) return;
            }
            performAction(action.action, action.compName, on);
        }
    }

    /**
     *  Performs the requested action.
     * 
     *  @param  action  The action
     *  @param  compName  The compressor name
     *  @param  on  Whether to turn on
     */
    private void performAction(RefrigAction.Action action, String compName, boolean on)
    {
        //LOG.log(Level.INFO, "Action requested: action = ({0}, {1}), on = {2}", new Object[]{action, compName, on});
        switch (action) {
        case STOP_ALL_COLD_COMPS:
            for (String name : coldComps.keySet()) {
                if (on || active[RefrigAction.Action.STOP_COLD_COMP.ordinal()].get(name).isEmpty()) {
                    coldComps.get(name).setPlateTempLow(on);
                }
            }
            break;
        case STOP_COLD_COMP:
            if (on || active[RefrigAction.Action.STOP_ALL_COLD_COMPS.ordinal()].get("").isEmpty()) {
                coldComps.get(compName).setPlateTempLow(on);
            }
            break;
        case STOP_ALL_CRYO_COMPS:
            for (String name : cryoComps.keySet()) {
                if (on || active[RefrigAction.Action.STOP_CRYO_COMP.ordinal()].get(name).isEmpty()) {
                    cryoComps.get(name).setPlateTempLow(on);
                }
            }
            break;
        case STOP_CRYO_COMP:
            if (on || active[RefrigAction.Action.STOP_ALL_CRYO_COMPS.ordinal()].get("").isEmpty()) {
                cryoComps.get(compName).setPlateTempLow(on);
            }
            break;
        }
    }

    /**
     *  Removes an originating agent.
     * 
     *  @param  origin  The agent name
     */
    private void removeOrigin(String origin)
    {
        if (clearOrigins.contains(origin)) {
            origins.remove(origin);
            clearOrigins.remove(origin);
        }
        else {
            clearOrigins.add(origin);
        }
    }

}

==> org-lsst-ccs-subsystem-refrig-main-4.4.1-sources.jar: org/lsst/ccs/subsystem/refrig/RefrigActionManager.java <==
package org.lsst.ccs.subsystem.refrig;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.Agent;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.bus.data.Alert;
import org.lsst.ccs.bus.data.RaisedAlertHistory;
import org.lsst.ccs.bus.states.AlertState;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.messaging.AgentPresenceListener;
import org.lsst.ccs.services.AgentPropertiesService;
import org.lsst.ccs.services.MessagingService;
import org.lsst.ccs.services.alert.AlertEvent;
import org.lsst.ccs.services.alert.AlertListener;
import org.lsst.ccs.services.alert.AlertService;
import org.lsst.ccs.subsystem.refrig.data.RefrigAction;

/**
 * This class manages the refrig actions in response to Alert payloads.
 *
 * It uses the AgentPresenceManager to detect which Agents on the buses might be
 * sending Alert payloads to trigger actions.
 *
 * @author The LSST CCS Team
 */
public class RefrigActionManager implements HasLifecycle, AlertListener, AgentPresenceListener {

    private static final Logger LOG = Logger.getLogger(RefrigActionManager.class.getName());

    @LookupField(strategy = LookupField.Strategy.TOP)
    private Agent agent;
    @LookupField(strategy = LookupField.Strategy.TREE)
    private AgentPropertiesService propertiesService;
    @LookupField(strategy = LookupField.Strategy.TREE)
    private AlertService alertService;
    @LookupField(strategy = LookupField.Strategy.TREE)
    private final Map<String, ColdCompressor> coldComps = new HashMap<>();
    @LookupField(strategy = LookupField.Strategy.TREE)
    private final Map<String, CryoCompressor> cryoComps = new HashMap<>();

    private final Set<String> origins = new ConcurrentSkipListSet<>();  // Client (originating) agents
    private final Set<String> clearOrigins = new ConcurrentSkipListSet<>();  // Agents to be cleared when disconnect processed
    private final Map<String, Set<String>>[] active = new Map[RefrigAction.Action.values().length];  // Active actions with agents

    /**
     *  Constructor.
     */
    public RefrigActionManager()
    {
        for (int j = 0; j < active.length; j++) {
            active[j] = new HashMap<>();
        }
    }

    /**
     *  Build phase.
     */
    @Override
    public void build()
    {
        active[RefrigAction.Action.STOP_ALL_COLD_COMPS.ordinal()].put("", new ConcurrentSkipListSet<>());
        for (String name : coldComps.keySet()) {
            active[RefrigAction.Action.STOP_COLD_COMP.ordinal()].put(name, new ConcurrentSkipListSet<>());
        }
        active[RefrigAction.Action.STOP_ALL_CRYO_COMPS.ordinal()].put("", new ConcurrentSkipListSet<>());
        for (String name : cryoComps.keySet()) {
            active[RefrigAction.Action.STOP_CRYO_COMP.ordinal()].put(name, new ConcurrentSkipListSet<>());
        }
    }

    /**
     *  Init phase.
     */
    @Override
    public void init()
    {
        // Set a property to define agent as MPM action manager.
        propertiesService.setAgentProperty(RefrigAction.getManagerName(), "");
        // Start alert listening
        alertService.startStatusAlertListening((info) -> {
            return info.isAgentWorkerOrService();
        });
        // Agent presence listener
        agent.getAgentService(MessagingService.class).getMessagingAccess().getAgentPresenceManager().addAgentPresenceListener(this);
    }

    /**
     *  Start phase.
     */
    @Override
    public void start()
    {
        //Alert listener responsible to listen for Alert emergency payloads
        //published by the origins maintained by the OriginManager
        alertService.addListener(this);
    }

    /**
     *  Shutdown phase.
     */
    @Override
    public void shutdown()
    {
        //Remove agent presence and alert listeners
        agent.getAgentService(MessagingService.class).getMessagingAccess().getAgentPresenceManager().removeAgentPresenceListener(this);
        alertService.removeListener(this);
    }

    /**
     *  Agent connection handler.
     * 
     *  @param  agents  Connecting agents
     */
    @Override
    public void connecting(AgentInfo... agents)  // Must use this, not "connected"
    {
        for (AgentInfo ai : agents) {
            if (ai.hasAgentProperty(RefrigAction.getClientName())) {
                LOG.log(Level.INFO, "Adding agent {0} to list of origins for payload {1}",
                        new Object[]{ai.getName(), RefrigAction.getName()});
                origins.add(ai.getName());
                clearOrigins.remove(ai.getName());
            }
        }
    }

    /**
     *  Agent disconnection handler.
     * 
     *  @param  agents  Disconnected agents
     */
    @Override
    public void disconnected(AgentInfo... agents)
    {
        for (AgentInfo ai : agents) {
            if (ai.hasAgentProperty(RefrigAction.getClientName())) {
                LOG.log(Level.INFO, "Removing agent {0} from list of origins for payload {1}",
                        new Object[]{ai.getName(), RefrigAction.getName()});
                removeOrigin(ai.getName());
            }
        }
    }

    /**
     *  Raised alert handler.
     * 
     *  @param  event  The alert event
     */
    @Override
    public void onAlert(AlertEvent event)
    {
        String origin = event.getSource();
        //LOG.log(Level.INFO, "onAlert called: origin = {0}, type = {1}", new Object[]{origin, event.getType()});
        if (origins.contains(origin)) {
            switch (event.getType()) {
            case AGENT_CONNECTION:
                for (RaisedAlertHistory alertHistory : event.getSummary().getAllRaisedAlertHistories()) {
                    processAlert(origin, alertHistory.getLatestAlert(), alertHistory.getLatestAlertState());                    
                }
                break;
            case AGENT_DISCONNECTION:
                removeOrigin(origin);
                for (RefrigAction.Action action : RefrigAction.Action.values()) {
                    for (Map.Entry<String, Set<String>> entry : active[action.ordinal()].entrySet()) {
                        if (entry.getValue().remove(origin) && entry.getValue().isEmpty()) {
                            performAction(action, entry.getKey(), false);
                        }
                    }
                }
                break;
            case ALERT_RAISED:
                processAlert(origin, event.getAlert(), event.getLevel());
                break;
            }
        }

    }

    /**
     *  Processes an alert.
     * 
     *  @param  origin  Name of originating agent
     *  @param  alert  The alert
     *  @param  state  The alert state (ALARM, WARNING, NOMINAL)
     */
    private synchronized void processAlert(String origin, Alert alert, AlertState state)
    {
        RefrigAction action = RefrigAction.getData(alert);
        if (action != null) {
            LOG.log(Level.INFO, "Received {0} level alert from {1}: action = ({2}, {3})",
                    new Object[]{state, origin, action.action, action.compName});
            Set orgSet = active[action.action.ordinal()].get(action.compName);
            if (orgSet == null) return;
            boolean on = state == AlertState.ALARM;
            if (on) {
                orgSet.add(origin);
            }
            else {
                orgSet.remove(origin);
                if (!orgSet.isEmpty()) return;
            }
            performAction(action.action, action.compName, on);
        }
    }

    /**
     *  Performs the requested action.
     * 
     *  @param  action  The action
     *  @param  compName  The compressor name
     *  @param  on  Whether to turn on
     */
    private void performAction(RefrigAction.Action action, String compName, boolean on)
    {
        //LOG.log(Level.INFO, "Action requested: action = ({0}, {1}), on = {2}", new Object[]{action, compName, on});
        switch (action) {
        case STOP_ALL_COLD_COMPS:
            for (String name : coldComps.keySet()) {
                if (on || active[RefrigAction.Action.STOP_COLD_COMP.ordinal()].get(name).isEmpty()) {
                    coldComps.get(name).setPlateTempLow(on);
                }
            }
            break;
        case STOP_COLD_COMP:
            if (on || active[RefrigAction.Action.STOP_ALL_COLD_COMPS.ordinal()].get("").isEmpty()) {
                coldComps.get(compName).setPlateTempLow(on);
            }
            break;
        case STOP_ALL_CRYO_COMPS:
            for (String name : cryoComps.keySet()) {
                if (on || active[RefrigAction.Action.STOP_CRYO_COMP.ordinal()].get(name).isEmpty()) {
                    cryoComps.get(name).setPlateTempLow(on);
                }
            }
            break;
        case STOP_CRYO_COMP:
            if (on || active[RefrigAction.Action.STOP_ALL_CRYO_COMPS.ordinal()].get("").isEmpty()) {
                cryoComps.get(compName).setPlateTempLow(on);
            }
            break;
        }
    }

    /**
     *  Removes an originating agent.
     * 
     *  @param  origin  The agent name
     */
    private void removeOrigin(String origin)
    {
        if (clearOrigins.contains(origin)) {
            origins.remove(origin);
            clearOrigins.remove(origin);
        }
        else {
            clearOrigins.add(origin);
        }
    }

}
